import 'bin_validate.dart';
import 'predefined_amount.dart';
import 'prefill.dart';
import 'transaction.dart';

class OperatorCredentials extends PaydalaOperator {
  CredentialsPayload payload;

  OperatorCredentials({
    required this.payload,
    String? clientId,
    int? categoryId,
    String? region,
    String? timestamp,
  }) : super(
          clientId: clientId,
          categoryId: categoryId,
          region: region,
          timestamp: timestamp,
        );
  @override
  Map<String, dynamic> toJson() => {
        'payload': payload.toJson(),
        'clientId': clientId,
        'categoryId': categoryId,
        'region': region,
        'timestamp': timestamp,
      };
  factory OperatorCredentials.fromJson(Map<String, dynamic> json) {
    return OperatorCredentials(
      payload: CredentialsPayload.fromJson(json['payload']),
      clientId: json['client_id'],
      categoryId: json['category_id'].toInt(),
      region: json['region'],
      timestamp: json['timestamp'],
    );
  }
}

class PaydalaOperator {
  String? clientId;
  int? categoryId;
  String? region;
  String? timestamp;

  PaydalaOperator({
    this.clientId,
    this.categoryId,
    this.region,
    this.timestamp,
  });

  Map<String, dynamic> toJson() => {
        'clientId': clientId,
        'categoryId': categoryId,
        'region': region,
        'timestamp': timestamp,
      };
}

class CredentialsPayload {
  String requestId;
  String? customerId;
  PredefinedAmount? predefinedAmount;
  Prefill? defaultUser;
  String redirectUrl;
  bool? isWebView;
  bool? isiFrame;
  TransactionType transactionType;
  UserFlow? userFlow;

  CredentialsPayload({
    required this.requestId,
    this.customerId,
    this.predefinedAmount,
    this.defaultUser,
    required this.redirectUrl,
    this.isWebView,
    this.isiFrame,
    required this.transactionType,
    this.userFlow,
  });

  Map<String, dynamic> toJson() => {
        'requestId': requestId,
        'customerId': customerId,
        if (predefinedAmount != null)
          'predefinedAmount': predefinedAmount?.toJson(),
        'defaultUser': defaultUser?.toJson(),
        'redirectUrl': redirectUrl,
        'isWebView': isWebView,
        if (isiFrame != null) 'isiFrame': isiFrame,
        'transactionType': transactionType.toString().split('.').last,
        'userFlow': userFlow?.toJson(),
      };

  factory CredentialsPayload.fromJson(Map<String, dynamic> json) {
    TransactionType transactionTypeFromJson(String transactionType) {
      return TransactionType.values.firstWhere(
        (e) => e.toString() == 'TransactionType.$transactionType',
        orElse: () =>
            throw Exception('Unknown TransactionType: $transactionType'),
      );
    }

    return CredentialsPayload(
      requestId: json['requestId'],
      customerId: json['customerId'],
      predefinedAmount: json['predefinedAmount'] != null
          ? PredefinedAmount.fromJson(json['predefinedAmount'])
          : null,
      defaultUser: Prefill.fromJson(json['defaultUser']),
      redirectUrl: json['redirectUrl'],
      isWebView: json['isWebView'],
      isiFrame: json['isiFrame'] as bool?,
      transactionType: transactionTypeFromJson(json['transactionType']),
      userFlow:
          json['userFlow'] != null ? UserFlow.fromJson(json['userFlow']) : null,
    );
  }
}

class UserFlow {
  bool? guestOnly;
  CardValidation? cardValidation;

  UserFlow({
    this.guestOnly,
    this.cardValidation,
  });
  Map<String, dynamic> toJson() => {
        'guestOnly': guestOnly,
        if (cardValidation != null) 'cardValidation': cardValidation?.toJson(),
      };

  factory UserFlow.fromJson(Map<String, dynamic> json) {
    return UserFlow(
      guestOnly: json['guestOnly'],
      cardValidation: json['cardValidation'] != null
          ? CardValidation.fromJson(json['cardValidation'])
          : null,
    );
  }
}

class CardValidation {
  List<CardType>? types;
  List<CardBrand>? brands;

  CardValidation({
    this.types,
    this.brands,
  });
  Map<String, dynamic> toJson() => {
        'types': types,
        'brands': brands,
      };

  factory CardValidation.fromJson(Map<String, dynamic> json) {
    CardType cardTypeFromJson(String cardType) {
      return CardType.values.firstWhere(
        (e) => e.toString() == 'CardType.$cardType',
        orElse: () => throw Exception('Unknown CardType: $cardType'),
      );
    }

    CardBrand cardBrandFromJson(String cardBrand) {
      return CardBrand.values.firstWhere(
        (e) => e.toString() == 'CardBrand.$cardBrand',
        orElse: () => throw Exception('Unknown CardBrand: $cardBrand'),
      );
    }

    return CardValidation(
      types: json['types'] != null
          ? (json['types'] as List).map((i) => cardTypeFromJson(i)).toList()
          : null,
      brands: json['brands'] != null
          ? (json['brands'] as List).map((i) => cardBrandFromJson(i)).toList()
          : null,
    );
  }
}
