import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_threatmetrix_module/flutter_threatmetrix_module.dart';
import 'package:paydala_flutter_sdk/webchannel_payload.dart';
import 'package:webview_flutter_android/webview_flutter_android.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:url_launcher/url_launcher.dart';

class WebViewExample extends StatefulWidget {
  const WebViewExample({Key? key, required this.url}) : super(key: key);
  final String url;

  @override
  State<WebViewExample> createState() => _WebViewExampleState();
}

class _WebViewExampleState extends State<WebViewExample> {
  late final WebViewController controller;

  @override
  void initState() {
    super.initState();
    controller = WebViewController()
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..loadRequest(Uri.parse(widget.url))
      ..setNavigationDelegate(
        NavigationDelegate(
          onProgress: (int progress) {
            // Update loading bar.
          },
          onPageStarted: (String url) {},
          onPageFinished: (String url) {},
          onWebResourceError: (WebResourceError error) {},
          onNavigationRequest: (NavigationRequest request) {
            if (request.url.contains('terms') ||
                request.url.contains('privacy') ||
                request.url.contains('dwolla')) {
              launchUrl(Uri.parse(request.url));
              return NavigationDecision.prevent;
            } else if (request.url.contains('paydala')) {
              return NavigationDecision.navigate;
            } else{
              launchUrl(Uri.parse(request.url));
              return NavigationDecision.prevent;
            }
          },
        ),
      )
      ..addJavaScriptChannel(
        'PayDala',
        onMessageReceived: (JavaScriptMessage message) {
          Map<String, dynamic> parsedJson = jsonDecode(message.message);
          WebchannelPayload payload = WebchannelPayload.fromJson(parsedJson);
          if (payload.type == 'closePaydala') {
            Navigator.pop(context);
          }
          if (payload.type == 'sessionCreated' && payload.sessionId != null) {
            getProfilingData(payload.sessionId!);
          }
        },
      );

    final platformController = controller.platform;
    if (platformController is AndroidWebViewController) {
      platformController.setGeolocationPermissionsPromptCallbacks(
        onShowPrompt: (request) async {
          // request location permission
          final locationPermissionStatus =
              await Permission.locationWhenInUse.request();

          // return the response
          return GeolocationPermissionsResponse(
            allow: locationPermissionStatus == PermissionStatus.granted,
            retain: false,
          );
        },
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SafeArea(
        child: WebViewWidget(
          controller: controller,
        ),
      ),
    );
  }

  Future<void> getProfilingData(String sessionId) async {
    String profileData = 'No Data';
    String? error;
    String orgId = '52whdrrv';
    String fpServer = 'h-sdk.online-metrix.net';
    print('SessionId: $sessionId');

    try {
      final result = await FlutterThreatmetrixModule()
              .getProfilingData(sessionId, orgId, fpServer) ??
          'No Data';
      profileData = result;
    } catch (e) {
      error = e.toString();
    }
  }
}
